#
# coursebuilder
#

from typing import Any
from argparse import ArgumentParser
from pathlib import Path
import yaml

from coursebuilder.schema import Schema


class Course:
    i18n_name: str = "lang.{}.yaml"
    mod_name: str = "mod.yaml"

    def __init__(self, *, path: Path) -> None:
        with open(path) as f:
            # resolve rest of bundle
            self.__data = yaml.load(f, Loader=yaml.Loader)
            # load i18n overlays
            self.__i18n = {
                f"{str(p).split('.')[1]}": yaml.load(open(p), Loader=yaml.Loader)
                for p in path.parent.glob(Course.i18n_name.format("*"))
            }

    def __getitem__(self, name: str, /) -> Any:
        """this overload magic function takes a key and a language code like my_key.en or my_key.de"""
        s = name.split(".")

        if len(s) != 2:
            raise ValueError(
                "query with item selector requires form ['key.lang'] alternative is ['key.*']"
            )
        return (
            self.__i18n[s[-1]][s[0]]
            if s[-1] in self.__i18n.keys() and s[0] in self.__i18n[s[-1]].keys()
            else self.__data[s[0]]
        )

    def __str__(self):
        return f"data:{self.__data}\ni18n:{self.__i18n}"


class StudyCourse:
    def __init__(self, *, path: Path) -> None:
        self.path = path

        with open(path) as f:
            self.__data = yaml.load(f, Loader=yaml.Loader)

        self.courses: dict[str, Course] = {
            f"{c}": Course(path=self.path.parent / c / Course.mod_name)
            for c in self.__data["courses"]
        }

    def __getattr__(self, name: str, /) -> Any:
        return self.__data[name] if self.__data else None

    def __str__(self):
        return f"path: {self.path}\ndata: {self.__data}"


def main():
    # parser
    parser = ArgumentParser(description="versatile curricula generator")

    # loading mode for internal database
    parser.add_argument(
        "-i",
        "--input",
        type=str,
        help="course file with definition of the course",
    )

    parser.add_argument(
        "-s",
        "--schema",
        type=str,
        help="schema to validate against",
    )
    parser.add_argument(
        "-f",
        "--fields",
        help="Fields to be used, the table will be build accordingly",
        action="extend",
        nargs="+",
        type=str,
    )
    parser.add_argument(
        "-c",
        "--create",
        help="Fields to be used, the table will be build accordingly",
        type=str,
    )

    # get arguments
    args = parser.parse_args()

    # with input
    if args.input and args.schema:
        with open(args.schema) as f_schema:
            schema = Schema(schema=yaml.load(f_schema, Loader=yaml.Loader))  # Schema
            sc = StudyCourse(path=(Path(".") / args.input).absolute())  # Database

            actual_fields = args.fields if args.fields else schema.keys()

            for field in actual_fields:
                for lang in sc.languages:
                    for shortcode, course in sc.courses.items():
                        print(
                            field,
                            "@",
                            shortcode,
                            ".",
                            lang,
                            "\n",
                            course[f"{field}.{lang}"],
                        )

    elif args.schema and args.create:
        schema = Schema(
            schema=yaml.load(open(args.schema), Loader=yaml.Loader)
        )  # Schema
        print(schema.fields())
        print(schema.types())
        print(schema.facets())


# run as main
if __name__ == "__main__":
    main()
